package io.r2u.flutter_r2u

import android.content.Intent
import android.net.Uri
import android.app.Activity
import android.content.Context
import androidx.annotation.NonNull

import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodChannel.Result
import com.google.ar.core.ArCoreApk


/** FlutterR2uPlugin */
class FlutterR2uPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private lateinit var channel : MethodChannel
  private var activity: Activity? = null
  private var applicationContext: Context? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "flutter_r2u")
    channel.setMethodCallHandler(this)

    applicationContext = flutterPluginBinding.getApplicationContext()
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "deviceSupportsAR") {
      val supportsAR = deviceSupportsAR()
      result.success(supportsAR)
    } else if (call.method == "openAR") {
      val glb: String = call.argument("glb")!!
      val resize: Boolean = ifcall.argument("resize")!!
      val placement: String = call.argument("placement")!!
      val title: String = call.argument("name")!!
      val sound: String = call.argument("sound")!!
      val shareText: String = call.argument("shareText")!!
      val link: String = call.argument("link")!!
      openAR(glb, resize, placement, title, sound, shareText, link)
      result.success(true)
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null)

    activity = null
    applicationContext = null
  }

  fun openAR(
        glb: String,
        resize: Boolean,
        placement: String,
        title: String,
        sound: String,
        shareText: String,
        link: String
  ) {
    val intentUri = Uri.parse("https://arvr.google.com/scene-viewer/1.2").buildUpon()
            .appendQueryParameter("file", glb)
            .appendQueryParameter("mode", "ar_preferred")
            .appendQueryParameter("disable_occlusion", "true")

    if (resize == false) {
        intentUri.appendQueryParameter("resizable", "false")
    }
    if (placement == "wall") {
        intentUri.appendQueryParameter("enable_vertical_placement", "true")
    }
    if (sound != "") {
        intentUri.appendQueryParameter("sound", sound)
    }
    if (link != "") {
        intentUri.appendQueryParameter("link", link)
    }
    if (title != "") {
        intentUri.appendQueryParameter("title", title)
    }
    if (shareText != "") {
        intentUri.appendQueryParameter("share_text", shareText)
    }

    val sceneViewerIntent = Intent(Intent.ACTION_VIEW)
    sceneViewerIntent.setData(intentUri.build())
    sceneViewerIntent.setPackage("com.google.ar.core")
    sceneViewerIntent.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK)

    if (activity != null) {
      activity?.startActivity(sceneViewerIntent)
    } else {
      applicationContext?.startActivity(sceneViewerIntent)
    }
  }

  // https://developers.google.com/ar/develop/java/enable-arcore#check_supported
  fun deviceSupportsAR(): Boolean {
    val context = if (activity != null) activity?.getApplicationContext() else applicationContext
    val availability = ArCoreApk.getInstance().checkAvailability(context)
    return availability.isSupported()
  }

  /**
   * See
   * https://stackoverflow.com/questions/59887901/get-activity-reference-in-flutter-plugin
   * https://pub.dev/packages/android_intent_plus
   * https://github.com/fluttercommunity/plus_plugins/blob/main/packages/android_intent_plus/android/src/main/java/dev/fluttercommunity/plus/androidintent/AndroidIntentPlugin.java
   * https://github.com/matteocrippa/flutter-nfc-reader/blob/master/android/src/main/kotlin/it/matteocrippa/flutternfcreader/FlutterNfcReaderPlugin.kt
   */

  override public fun onAttachedToActivity(activityPluginBinding: ActivityPluginBinding) {
    activity = activityPluginBinding.activity
  }

  override fun onDetachedFromActivityForConfigChanges() {
    onDetachedFromActivity()
  }

  override fun onReattachedToActivityForConfigChanges(activityPluginBinding: ActivityPluginBinding) {
    onAttachedToActivity(activityPluginBinding)
  }

  override fun onDetachedFromActivity() {
    activity = null
  }
}
