import 'dart:async';

import 'package:flutter_r2u/product.dart';

import 'package:flutter/services.dart';
import 'package:flutter_r2u/state.dart';

class R2U {
  static const MethodChannel _channel = const MethodChannel('flutter_r2u');

  /// Initializes the R2U SDK
  /// This method must be called before the use of other functions
  static Future<void> init({customerId: String}) async {
    print("R2U.init " + customerId);
    state.init = true;
    state.customerId = customerId;
  }

  /// Checks whether the user device supports Augmented Reality
  /// This method must be used before showing an AR call to action, such as a button, to a user
  static Future<bool> deviceSupportsAR() async {
    print("R2U.deviceSupportsAR");
    final bool supportsAR = await _channel.invokeMethod('deviceSupportsAR');
    return supportsAR;
  }

  /// Checks whether the product SKU is active and running on the R2U platform
  /// This method must be used before showing an AR call to action, such as a button, for the specific product
  static Future<bool> isActive(String sku) async {
    print("R2U.isActive " + sku);
    final p = await product(sku);
    return p.isActive;
  }

  /// Open the Augmented Reality experience using ARKit for iOS and ARCore for Android
  static Future<void> openAR({sku: String, resize = false}) async {
    print("R2U.openAR " + sku);
    final p = await product(sku);
    await _channel.invokeMethod('openAR', {
      "glb": p.glb,
      "usdz": p.usdz,
      "resize": p.resize,
      "placement": p.placement,
      "name": p.name,
      "sound": p.sound,
      "link": p.pdpUrl,
      "shareText": '',
    });
  }

  /// Returns the 3D URL to display the product on a webview
  static Future<String> getLink(String sku) async {
    print("R2U.getLink " + sku);
    if (state.init == false) {
      throw Exception("R2U — Not initialized");
    }
    final p = await product(sku);
    final url = 'https://viewer.r2u.io/3d/${Uri.encodeComponent(p.glb)}';
    return url;
  }
}
