import 'dart:convert';

import 'package:flutter_r2u/state.dart';
import 'package:http/http.dart' as http;

class Product {
  final String glb;
  final String usdz;
  final bool isActive;
  final String name;
  final String placement;
  final bool resize;
  final String pdpUrl;
  final String sound;
  final String? modelId;
  final String? manufacturerId;

  Product({
    required this.glb,
    required this.usdz,
    required this.isActive,
    required this.name,
    required this.placement,
    required this.resize,
    required this.pdpUrl,
    required this.sound,
    this.modelId,
    this.manufacturerId,
  });
}

Future<Product> product(String sku) async {
  if (state.init == false) {
    throw Exception("R2U — Not initialized");
  }
  if (state.products[sku] != null) {
    return state.products[sku] as Product;
  }

  final query = {
    'product.sku': sku,
    'asset.types': ['GLB', 'USDZ'].join(','),
    'product_review.status': 'RUNNING',
  };

  final response = await http.get(Uri.https(
      'api.r2u.io', '/customers/${state.customerId}/products/assets/', query));

  if (response.statusCode == 200) {
    List<dynamic> json = jsonDecode(response.body);
    print(json);
    final customerProduct = json[0];
    final assets = customerProduct['assets'] ?? [];
    final productReviews = customerProduct['product_reviews'] ?? [];
    final glbAsset = assets.firstWhere((asset) => asset['type'] == 'GLB');
    final usdzAsset = assets.firstWhere((asset) => asset['type'] == 'USDZ');

    final product = Product(
      glb: glbAsset != null ? glbAsset['url'] : '',
      usdz: usdzAsset != null ? usdzAsset['url'] : '',
      isActive: productReviews.length > 0,
      name: customerProduct['name'],
      placement: customerProduct['placement'] ?? 'floor',
      resize: customerProduct['resize'] ?? false,
      pdpUrl: customerProduct['pdpUrl'] ?? '',
      sound: customerProduct['soundUrl'] ?? '',
      modelId: customerProduct['modelId'],
      manufacturerId: customerProduct['manufacturerId'],
    );

    state.products[sku] = product;
    return product;
  } else {
    throw Exception('R2U — Failed to load product');
  }
}
